@@ -3,10 +3,11 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
 import sys
 
+from ..compat import USE_WORDCODE
 from . import compiler_common as cmn
 from .enaml_ast import Module
 from .enamldef_compiler import EnamlDefCompiler
@@ -128,7 +129,14 @@ from .template_compiler import TemplateCompiler
 #     declarative method overrides.
 # 22 : Update the syntax of arrow functions - 5 May 2014
 #     This updates the arrow functions to use "=>" instead of "->".
-COMPILER_VERSION = 22
+# 23 : Support for Python 3 and inlining of comprehensions.
+# 24 : Call comprehension functions in the proper scope rather than inlining
+# 25 : Support for Python 3.6
+# 26 : Wrap functions defined inside operators or declarative function to call
+#      them with their scope of definition. This allows to handle properly
+#      comprehensions and lambdas. Also ensure that we compile the body of the
+#      :: operator as a function to properly handle closure.
+COMPILER_VERSION = 26
 
 
 # Code that will be executed at the top of every enaml module
@@ -166,11 +174,6 @@ class EnamlCompiler(cmn.CompilerBase):
         """
         assert isinstance(node, Module), 'invalid node'
 
-        # Protect against unicode filenames, which are incompatible
-        # with code objects created via types.CodeType
-        if isinstance(filename, unicode):
-            filename = filename.encode(sys.getfilesystemencoding())
-
         # Create the compiler and generate the code.
         compiler = cls(filename=filename)
         return compiler.visit(node)
@@ -214,6 +217,7 @@ class EnamlCompiler(cmn.CompilerBase):
         cg = self.code_generator
         code = EnamlDefCompiler.compile(node, cg.filename)
         cg.load_const(code)
+        cg.load_const(None)  # XXX better qualified name
         cg.make_function()
         cg.call_function()
         cg.store_global(node.typename)
@@ -246,10 +250,21 @@ class EnamlCompiler(cmn.CompilerBase):
                     cg, param.default.ast, node.name, param.lineno, set()
                 )
 
+            # Under Python 3.6+ default positional arguments are passed as a
+            # single tuple and MAKE_FUNCTION is passed the flag 0x01 to
+            # indicate that there is default positional arguments.
+            if USE_WORDCODE:
+                cg.build_tuple(len(node.parameters.keywords))
+
             # Generate the template code and function
             code = TemplateCompiler.compile(node, cg.filename)
             cg.load_const(code)
-            cg.make_function(len(node.parameters.keywords))
+
+            # Under Python 3 function have a qualified name
+            # XXX improve qualified name
+            cg.load_const(None)
+            cg.make_function(0x01 if USE_WORDCODE else
+                             len(node.parameters.keywords))
 
             # Load and call the helper which will build the template
             cmn.load_helper(cg, 'make_template', from_globals=True)
