@@ -1,14 +1,15 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2013, Nucleic Development Team.
+# Copyright (c) 2013-2018, Nucleic Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
 from atom.api import Typed, Bool
 
 from enaml.qt.QtCore import Qt, QMargins, QPoint, QRect, QEvent, Signal
-from enaml.qt.QtGui import QApplication, QLayout, QIcon, QCursor
+from enaml.qt.QtGui import QIcon, QCursor
+from enaml.qt.QtWidgets import QApplication, QLayout
 
 from .event_types import QDockItemEvent, DockItemUndocked
 from .q_dock_area import QDockArea
@@ -108,7 +109,7 @@ class QDockContainer(QDockFrame):
     topLevelChanged = Signal(bool)
 
     #: A signal emitted when the container is alerted.
-    alerted = Signal(unicode)
+    alerted = Signal(str)
 
     class FrameState(QDockFrame.FrameState):
         """ A private class for managing container drag state.
@@ -490,7 +491,7 @@ class QDockContainer(QDockFrame):
             parent = parent.parent()
 
     def parentDockTabWidget(self):
-        """ Get the parent dock area of the container.
+        """ Get the parent dock tab of the container.
 
         Returns
         -------
@@ -500,9 +501,12 @@ class QDockContainer(QDockFrame):
 
         """
         parent = self.parent()
+        parent_area = self.parentDockArea()
         while parent is not None:
             if isinstance(parent, QDockTabWidget):
                 return parent
+            elif parent is parent_area:
+                return None
             parent = parent.parent()
 
     def unplug(self):
