@@ -1,12 +1,11 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2013, Nucleic Development Team.
+# Copyright (c) 2013-2018, Nucleic Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
 
-
 #: The global dict of registered stylesheets. This dict should never be
 #: modified directly. Use the functions 'register_style_sheet' and
 #: 'get_style_sheet' instead.
@@ -30,7 +29,7 @@ def register_style_sheet(name, sheet):
     msg += "to style the Dock Area instead."
     warnings.warn(msg, FutureWarning, stacklevel=2)
 
-    assert isinstance(sheet, unicode), 'style sheet must a unicode string'
+    assert isinstance(sheet, str), 'style sheet must be a string'
     if name in _style_sheets:
         raise ValueError("'%s' style is already registered" % name)
     _style_sheets[name] = sheet
