@@ -1,14 +1,17 @@
 #------------------------------------------------------------------------------
-#  Copyright (c) 2012, Enthought, Inc.
-#  All rights reserved.
+# Copyright (c) 2013, Nucleic Development Team.
+#
+# Distributed under the terms of the Modified BSD License.
+#
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
 from abc import ABCMeta, abstractmethod
 
-from .qt.QtCore import Qt, QSize, QRect
-from .qt.QtGui import QLayout, QWidgetItem
+from .QtCore import Qt, QSize, QRect
+from .QtWidgets import QLayout, QWidgetItem
 
 
-class AbstractFlowWidget(object):
+class AbstractFlowWidget(object, metaclass=ABCMeta):
     """ An abstract base class which defines the interface for widgets
     which can be used in a QFlowLayout.
 
@@ -16,7 +19,6 @@ class AbstractFlowWidget(object):
     this class in order to use the QFlowLayout.
 
     """
-    __metaclass__ = ABCMeta
 
     @abstractmethod
     def layoutData(self):
@@ -318,7 +320,8 @@ class _LayoutRow(object):
                     y += delta_h
                 elif align == QFlowLayout.AlignCenter:
                     y += delta_h / 2
-            item.setGeometry(QRect(x, y, w, h))
+            item.setGeometry(QRect(x, int(round(y)),
+                                   int(round(w)), int(round(h))))
             return
 
         # Reversing the items reverses the layout direction. All of the
@@ -405,7 +408,8 @@ class _LayoutRow(object):
                     this_y = y + delta
                 elif align == QFlowLayout.AlignCenter:
                     this_y = y + delta / 2
-            item.setGeometry(QRect(curr_x, this_y, w, h))
+            item.setGeometry(QRect(int(round(curr_x)), int(round(this_y)),
+                                   int(round(w)), int(round(h))))
             curr_x += (w + space)
 
 
@@ -997,7 +1001,7 @@ class QFlowLayout(QLayout):
         # lie between their minimum height and desired height.
         height = rect.height()
         play_space = max(0, height - min_height)
-        diff_space = max(total_diff, 1) # Guard against divide by zero
+        diff_space = max(total_diff, 1)  # Guard against divide by zero
         layout_height = 0
         for row in rows:
             d = play_space * row.diff_height / diff_space
@@ -1069,7 +1073,7 @@ class QFlowLayout(QLayout):
         # which lie between their minimum width and desired width.
         width = rect.width()
         play_space = max(0, width - min_width)
-        diff_space = max(total_diff, 1) # Guard against divide by zero
+        diff_space = max(total_diff, 1)  # Guard against divide by zero
         layout_width = 0
         for col in cols:
             d = play_space * col.diff_width / diff_space
@@ -1115,4 +1119,3 @@ class _LayoutOptions(object):
 
     #: The vertical spacing between items or lines in the layout.
     v_spacing = 10
-
