@@ -1,16 +1,16 @@
 #------------------------------------------------------------------------------
-# Copyright (c) 2014, Nucleic Development Team.
+# Copyright (c) 2014-2017, Nucleic Development Team.
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
 from atom.api import Typed
 
 from enaml.application import Application, ProxyResolver
 
 from .QtCore import QThread
-from .QtGui import QApplication
+from .QtWidgets import QApplication
 
 from .q_deferred_caller import deferredCall, timedCall
 from .qt_factories import QT_FACTORIES
@@ -65,7 +65,7 @@ class QtApplication(Application):
         callback : callable
             The callable object to execute at some point in the future.
 
-        *args, **kwargs
+        args, kwargs
             Any additional positional and keyword arguments to pass to
             the callback.
 
@@ -85,7 +85,7 @@ class QtApplication(Application):
         callback : callable
             The callable object to execute at some point in the future.
 
-        *args, **kwargs
+        args, kwargs
             Any additional positional and keyword arguments to pass to
             the callback.
 
