@@ -3,11 +3,13 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
+from __future__ import print_function
+
 import datetime
 
-from atom.api import Atom, Unicode, Range, Bool, Value, Int, Tuple, observe
+from atom.api import Atom, Str, Range, Bool, Value, Int, Tuple, observe
 import enaml
 from enaml.qt.qt_application import QtApplication
 
@@ -16,9 +18,9 @@ class Person(Atom):
     """ A simple class representing a person object.
 
     """
-    last_name = Unicode()
+    last_name = Str()
 
-    first_name = Unicode()
+    first_name = Str()
 
     age = Range(low=0)
 
@@ -36,7 +38,24 @@ class Person(Atom):
             s = templ.format(
                 first=self.first_name, last=self.last_name, age=self.age,
             )
-            print s
+            print(s)
+
+    @observe('dob')
+    def update_age(self, change):
+        """ Update the person's age whenever their date of birth changes
+
+        """
+        # grab the current date time
+        now = datetime.datetime.utcnow()
+        # estimate the person's age within one year accuracy
+        age = now.year - self.dob.year
+        # check to see if the current date is before their birthday and
+        # subtract a year from their age if it is
+        if ((now.month == self.dob.month and now.day < self.dob.day)
+            or now.month < self.dob.month):
+                age -= 1
+        # set the persons age
+        self.age = age
 
 
 class Employer(Person):
@@ -44,7 +63,7 @@ class Employer(Person):
 
     """
     # The name of the company
-    company_name = Unicode()
+    company_name = Str()
 
 
 class Employee(Person):
@@ -59,11 +78,11 @@ class Employee(Person):
 
     # This method will be called automatically by atom when the
     # employee's phone number changes
-    def _phone_changed(self, val):
-        print 'received new phone number for %s: %s' % (self.first_name, val)
+    def _observe_phone(self, val):
+        print('received new phone number for %s: %s' % (self.first_name, val))
 
 
-if __name__ == '__main__':
+def main():
     # Create an employee with a boss
     boss_john = Employer(
         first_name='John', last_name='Paw', company_name="Packrat's Cats",
@@ -83,3 +102,6 @@ if __name__ == '__main__':
     view.show()
 
     app.start()
+
+if __name__ == '__main__':
+    main()
