@@ -3,9 +3,11 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
-from atom.api import Atom, Unicode, Range, Bool, observe
+from __future__ import unicode_literals, print_function
+
+from atom.api import Atom, Str, Range, Bool, observe
 
 import enaml
 from enaml.qt.qt_application import QtApplication
@@ -15,9 +17,9 @@ class Person(Atom):
     """ A simple class representing a person object.
 
     """
-    last_name = Unicode()
+    last_name = Str()
 
-    first_name = Unicode()
+    first_name = Str()
 
     age = Range(low=0)
 
@@ -33,10 +35,10 @@ class Person(Atom):
             s = templ.format(
                 first=self.first_name, last=self.last_name, age=self.age,
             )
-            print s
+            print(s)
 
 
-if __name__ == '__main__':
+def main():
     with enaml.imports():
         from person_view import PersonView
 
@@ -48,3 +50,6 @@ if __name__ == '__main__':
     view.show()
 
     app.start()
+
+if __name__ == '__main__':
+    main()
