@@ -3,7 +3,7 @@
 #
 # Distributed under the terms of the Modified BSD License.
 #
-# The full license is in the file COPYING.txt, distributed with this software.
+# The full license is in the file LICENSE, distributed with this software.
 #------------------------------------------------------------------------------
 """ A simple example plugin application.
 
@@ -11,14 +11,18 @@ This example serves to demonstrate the concepts described the accompanying
 developer crash source document.
 
 """
+import enaml
 from enaml.workbench.ui.api import UIWorkbench
 
 
-if __name__ == '__main__':
-    import enaml
+def main():
     with enaml.imports():
         from sample_plugin import SampleManifest
 
     workbench = UIWorkbench()
     workbench.register(SampleManifest())
     workbench.run()
+
+
+if __name__ == '__main__':
+    main()
